"""This function removes the background from spots traces.

Background value is different for each spot. It is extimated
averaging intensity values of pixels around the spots itself.
x, y coordinates are taken from the spots_trackes matrix (T, X, Y)
and the z coordinates is given by the maximum intensity in the row.
A 'cage' is built around the spots and intesities of the cage-pixels
are averaged as background value.
"""

import numpy as np
from skimage.measure import regionprops


class SpotsBackgroundEstimation:
    def __init__(self, spots_tracked, green4d, spt_id):

        z_tot, x_tot, y_tot  =  green4d[0, :, :, :].shape
        spt_mtx              =  (spots_tracked == spt_id)                                    # the spots is isolated from the matrix (T, X, Y)
        spt_prof             =  spt_mtx.sum(2).sum(1)                                        # its intensity profile
        bkg_values           =  np.zeros(np.append(spt_prof.size, 2))
        bkg_variab           =  np.zeros(spt_prof.size)
        bkg_cage             =  np.zeros(green4d.shape)

        for t in range(bkg_values.shape[0]):
            if spt_prof[t] != 0:                                                               # check the spot is present in the frame
                x_ctr, y_ctr  =  regionprops(spt_mtx[t].astype(np.int))[0]['centroid']
                x_ctr, y_ctr  =  int(round(x_ctr)), int(round(y_ctr))
                z_ctr         =  np.argmax(green4d[t, :, x_ctr, y_ctr])                        # z center coordinate

                bkg_vol    =  np.zeros(green4d[0, :, :, :].shape)
                z_min_ext  =  np.max([z_ctr - 5, 0])                                           # edges of the cage, internal and external. Controls for spots close to the borders
                z_max_ext  =  np.min([z_ctr + 6, z_tot])                                       # in zed the edge is smaller beacause in z the step is smaller than in x or y
                z_min_int  =  np.max([z_ctr - 3, 0])
                z_max_int  =  np.min([z_ctr + 4, z_tot])

                x_min_ext  =  np.max([x_ctr - 9, 0])
                x_max_ext  =  np.min([x_ctr + 10, x_tot])
                x_min_int  =  np.max([x_ctr - 7, 0])
                x_max_int  =  np.min([x_ctr + 8, x_tot])

                y_min_ext  =  np.max([y_ctr - 9, 0])
                y_max_ext  =  np.min([y_ctr + 10, y_tot])
                y_min_int  =  np.max([y_ctr - 7, 0])
                y_max_int  =  np.min([y_ctr + 8, y_tot])

                bkg_vol[z_min_ext:z_max_ext, x_min_ext:x_max_ext, y_min_ext:y_max_ext]  =  1                # cage definition (contains more than 1400 pixels)
                bkg_vol[z_min_int:z_max_int, x_min_int:x_max_int, y_min_int:y_max_int]  =  0
                bkg_cage[t]  =  np.copy(bkg_vol)
                bkg_bffr                                                                =  green4d[t, :, :, :] * bkg_vol
                bkg_values[t, 0]                                                        =  bkg_bffr[bkg_bffr != 0].mean()     # average value of the intensity in the cage walls
                bkg_values[t, 1]                                                        =  z_ctr                              # z coordinate of the spot
                bkg_variab[t]                                                           =  bkg_bffr[bkg_bffr != 0].std()      # standard deviation of the intensity in the cage walls


        self.bkg_values  =  bkg_values
        self.bkg_variab  =  bkg_variab
        self.bkg_cage  =  bkg_cage


